/************************************************************************/
/*  Copyright (c) 1997-2000 Nalante, Inc., 245 Falconer End, Edmonton,  */
/*  Alberta, Canada, T6R 2V6, Phone: (780) 432-5288, Fax: (780) 988-0184*/
/*  All Rights Reserved                                                 */
/*                                                                      */
/*  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Nalante, Inc.        */
/*  The copyright notice above does not evidence any                    */
/*  actual or intended publication of such source code.                 */
/*                                                                      */
/*  Unauthorized copy or distribution of this file is prohibited.       */
/************************************************************************/



#ifndef CMP_H
#define CMP_H

#include "config.h"

#define EQUAL_TO  0
#define GREATER_THAN 1
#define LESS_THAN  -1
#define DISJOINT -2
#define INTERSECT 2

inline int c_eq(int cmp_result) { return cmp_result==0;}
inline int c_leq(int cmp_result) { return cmp_result==-1 || cmp_result==0;}
inline int c_geq(int cmp_result) { return cmp_result==1 || cmp_result==0;}
inline int incomparable(int cmp_result)
{
  return cmp_result<-1 || cmp_result>1;
}

inline int cmp_val(const void* u, const void* v)
{
  if (u==0) {
    if (v==0) return EQUAL_TO;
    else return LESS_THAN;
  }
  else if (v==0) return GREATER_THAN;
  else if (u==v) return EQUAL_TO;
  else return INTERSECT;
}

inline int cmp_float(float u, float v)
{
  float diff = u-v;
  if (diff<0)
    return LESS_THAN;
  else if (diff>0)
    return GREATER_THAN;
  else
    return EQUAL_TO;
}

inline int cmp_double(double u, double v)
{
  double diff = u-v;
  if (diff<0)
    return LESS_THAN;
  else if (diff>0)
    return GREATER_THAN;
  else
    return EQUAL_TO;
}

inline int cmp_long(long u, long v)
{
  if (u<v)
    return LESS_THAN;
  else if (u>v)
    return GREATER_THAN;
  else
    return EQUAL_TO;
}

inline int cmp_unsigned_long(unsigned long u, unsigned long v)
{
  if (u<v)
    return LESS_THAN;
  else if (u>v)
    return GREATER_THAN;
  else
    return EQUAL_TO;
}

inline int cmp_int(int u, int v)
{
  if (u<v)
    return LESS_THAN;
  else if (u>v)
    return GREATER_THAN;
  else
    return EQUAL_TO;
}

inline int unify_cmp_result(int r1, int r2)
{
  if (r1==r2 || r2==EQUAL_TO) return r1;
  else if (r1==EQUAL_TO) return r2;
  else return INTERSECT;
}

class CLASS_DECLSPEC CmpResults {
  int _eq;
  int _gt;
  int _lt;
  int _intersect;
  int _disj;
public:
  void count(int cmp_result);
  int eq() { return _eq;}
  int gt() { return _gt;}
  int lt() { return _lt;}
  int intersect() { return _intersect;}
  int disj() { return _disj;}
  CmpResults() {
    _eq = 0;
    _gt = 0;
    _lt = 0;
    _intersect = 0;
    _disj = 0;
  }
};

#endif /* CMP_H */
