/************************************************************************/
/*  Copyright (c) 1997-2000 Nalante, Inc., 245 Falconer End, Edmonton,  */
/*  Alberta, Canada, T6R 2V6, Phone: (780) 432-5288, Fax: (780) 988-0184*/
/*  All Rights Reserved                                                 */
/*                                                                      */
/*  THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Nalante, Inc.        */
/*  The copyright notice above does not evidence any                    */
/*  actual or intended publication of such source code.                 */
/*                                                                      */
/*  Unauthorized copy or distribution of this file is prohibited.       */
/************************************************************************/




#ifndef FORALL_H
#define FORALL_H

#ifndef CONFIG_H
#include "config.h"
#endif

/* forall is a macro used to iterate through elements in a list. It
   four arguments. The first one the type of the elements. The second
   one is the variable that will be bound to the elements in turn. The
   third one is the type of the list. It is assumed that the iterator
   class of the type of list is defined and named <ListType>Iterator.
   The last one is a pointer to a list or a reference to a list
   depending on what kind of argument the constructor of the iterator
   requires. Here are two examples, let slist and alist be a Slist and
   an Alist respectively.

   forall (Ent, x, Alist, alist) {
     forall (Ent, y, Slist, &slist)
       use(x, y);
   }

   Note that forall macro expands to one and half statements. The first
   one statement is a declaration and initialization of a iterator. The
   half statement is a for loop without the body. Therefore,

   forall (Ent, x, Alist, alist)
     forall (Ent, y, Slist, &slist)
       use(x, y);

   will cause syntax error, because the body of the outer loop actually
   consists of two statements, even though it looks like one.
   Same thing can be said if forall is the body of other statements
   such as if, while, ...

   The compiler complains about the declaration of iterator if the macro
   appears directly in a case body. E.g.:

   case ONE:
     forall (Ent, y, Slist, &slist) use(y);
     break;

   The proper way deal with this situation is to use a block:

   case ONE: {
     forall (Ent, y, Slist, &slist) use(y);
     break;
   }
*/

#ifndef name2
#define name2(a, b) a ## b
#endif

class CLASS_DECLSPEC Iterator {
  int _updated;
public:
  int is_updated() { int u = _updated; _updated = 0; return u;}
  void updated(int u = 1) { _updated = u;}
  Iterator() { _updated = 0;}
};

#define forall(Type, x, ListType, list) \
name2(ListType,Iterator)<Type>  name2(x,_it)(list); \
for (Type x = name2(x,_it).current();!name2(x,_it).finished(); \
     x = ((name2(x,_it).is_updated())?name2(x,_it).current():name2(x,_it).next()))

#define foreach(Type, x, ListType, list) \
name2(ListType,Iterator) name2(x,_it)(list); \
for (Type x = (Type) name2(x,_it).current(); \
     !name2(x,_it).finished(); \
     (x = (Type) ((name2(x,_it).is_updated())?name2(x,_it).current():name2(x,_it).next())))

#define for_following(Type, x, ListType, outer) \
name2(ListType,FollowingIterator)<Type>  name2(x,_it)(name2(outer,_it)); \
for (Type x = name2(x,_it).current();!name2(x,_it).finished(); \
     x = ((name2(x,_it).is_updated())?name2(x,_it).current():name2(x,_it).next()))

#define SKIP_CURRENT(x) name2(x,_it).skip()
#define DELETE_CURRENT(x) name2(x,_it).del_current()
#define INSERT_BEFORE(x,y) name2(x,_it).insert_before_current(y)
#define INSERT_AFTER(x,y) name2(x,_it).insert_after_current(y)
#define DELETE_PARENT(x,y) name2(x,_it).del_parent(y)

#endif
